import sys
import datetime
import smtplib, ssl
import socket

from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

from de.p1st.exec_notify.lib import config


def sendMailOrWriteToFile(SUBJECT: str, BODY: str, informAboutLocalMail: bool = True):
    if informAboutLocalMail and _localMailExists():
        SUBJECT=f'{SUBJECT} | UNREAD LOCAL MAIL'
        BODY=f'[!] Note [!]\n' \
             f'There is local mail inside [file][maildir] that was not delivered previously! ' \
             f'Please read and then delete it to get rid of this warning.\n\n\n' \
             f'{BODY}'

    try:
        sendMail(SUBJECT=SUBJECT, BODY=BODY)
    except Exception as e:
        print(f'execNotify>> Could not send mail: {e}', file=sys.stderr)
        print(f'execNotify>> Writing to file instead ...', file=sys.stderr)

        # Instead, try to save the mail so that the user can read
        # it later when connected to this computer
        saveMail(SUBJECT=SUBJECT, BODY=BODY)


def sendMail(SUBJECT: str, BODY: str):
    """
    :throws Exception: If mail could not be sent
    """

    FROM = config.getFrom()
    TO = config.getTo()
    password = config.getPassword()

    # Create a secure SSL context
    context = ssl.create_default_context()

    host, port = config.getHostAndPort()
    with smtplib.SMTP_SSL(host=host, port=port, context=context) as server:
        server.login(FROM, password)
        message = MIMEMultipart()
        message["Subject"] = SUBJECT
        message["From"] = FROM
        message["To"] = TO
        message["Bcc"] = TO  # Recommended for mass emails
        # Turn plain/html message_body into plain/html MIMEText objects
        message.attach(MIMEText(BODY, "plain"))

        server.sendmail(FROM, TO, message.as_string())


def saveMail(SUBJECT: str, BODY: str):
    """
    This method does NOT throw exceptions
    """

    time = datetime.datetime.now()
    timeStr = time.strftime('%Y%m%d_%H%M%S')

    mailDir = config.getMailDir()
    mailFile = mailDir.joinpath(timeStr)

    try:
        # create parent directory if not existent
        mailDir.mkdir(parents=True, exist_ok=True)

        # append to file; create file if not existent
        with open(mailFile, "a") as f:
            f.write(f'{"=" * 20}\n=== date ===\n{timeStr}\n=== subject ===\n{SUBJECT}\n=== body ===\n{BODY}\n')
    except Exception as e:
        print(f'execNotify>> Could not write to file: {e}', file=sys.stderr)


def _localMailExists():
    """
    :return: True if local mail exists in maildir folder. Once the mail is read the user shall delete (or move) it.
    """
    mailDir = config.getMailDir()
    if not mailDir.exists():
        return False
    else:
        return len(list(mailDir.iterdir())) > 0
